/* $Id: foxio.c,v 1.1 2007/11/11 22:26:31 tat Exp $
 * KoanLogic - http://koanlogic.com */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <foxio/foxio.h>
#include "asm/etraxgpio.h"

#ifndef IO_SETGET_INPUT
#define IO_SETGET_INPUT   0x12
#endif

#ifndef IO_SETGET_OUTPUT
#define IO_SETGET_OUTPUT  0x13
#endif

#define PIN(port, bit) { port, bit, #port #bit, 0, 0 }
#define FOX_NA { NA, 0, "N/A", 0, 0 }

static const pin_t fox_pins_tmpl[] = {
    FOX_NA,         /* 0 */
    FOX_NA,         /* 1 */
    FOX_NA,         /* 2 */
    PIN(IOG, 22),   /* 3 */
    PIN(IOG, 23),   /* 4 */
    PIN(IOG, 20),   /* 5 */
    PIN(IOG, 21),   /* 6 */
    PIN(IOG, 18),   /* 7 */
    PIN(IOG, 19),   /* 8 */
    PIN(IOG, 16),   /* 9 */
    PIN(IOG, 17),   /* 10 */
    FOX_NA,         /* 11 */
    FOX_NA,         /* 12 */
    PIN(OG,  25),   /* 13 */
    FOX_NA,         /* 14 */
    FOX_NA,         /* 15 */
    FOX_NA,         /* 16 */
    FOX_NA,         /* 17 */
    FOX_NA,         /* 18 */
    FOX_NA,         /* 19 */
    FOX_NA,         /* 20 */
    PIN(IOG, 24),   /* 21 */
    FOX_NA,         /* 22 */
    FOX_NA,         /* 23 */
    FOX_NA,         /* 24 */
    FOX_NA,         /* 25 */
    FOX_NA,         /* 26 */
    FOX_NA,         /* 27 */
    FOX_NA,         /* 28 */
    FOX_NA,         /* 29 */
    FOX_NA,         /* 30 */
    FOX_NA,         /* 31 */
    FOX_NA,         /* 32 */
    FOX_NA,         /* 33 */
    FOX_NA,         /* 34 */
    FOX_NA,         /* 35 */
    FOX_NA,         /* 36 */
    FOX_NA,         /* 37 */
    PIN(IOA, 1),    /* 38 */
    FOX_NA,         /* 39 */
    FOX_NA,         /* 40 */
    FOX_NA,         /* 41 */
    FOX_NA,         /* 42 */
    FOX_NA,         /* 43 */
    FOX_NA,         /* 44 */
    FOX_NA,         /* 45 */
    FOX_NA,         /* 46 */
    FOX_NA,         /* 47 */
    FOX_NA,         /* 48 */
    FOX_NA,         /* 49 */
    FOX_NA,         /* 50 */
    FOX_NA,         /* 51 */
    FOX_NA,         /* 52 */
    PIN(IOG, 9),    /* 53 */
    PIN(IOG, 8),    /* 54 */
    PIN(IOG, 11),   /* 55 */
    PIN(IOG, 10),   /* 56 */
    PIN(IOG, 13),   /* 57 */
    PIN(IOG, 12),   /* 58 */
    PIN(IOG, 15),   /* 59 */
    PIN(IOG, 14),   /* 60 */
    PIN(OG,  2),    /* 61 */
    PIN(OG,  5),    /* 62 */
    PIN(OG,  1),    /* 63 */
    FOX_NA,         /* 64 */
    PIN(OG,  4),    /* 65 */
    PIN(OG,  3),    /* 66 */
    FOX_NA,         /* 67 */
    FOX_NA,         /* 68 */
    FOX_NA,         /* 69 */
    FOX_NA,         /* 70 */
    FOX_NA,         /* 71 */
    FOX_NA,         /* 72 */
    FOX_NA,         /* 73 */
    FOX_NA,         /* 74 */
    FOX_NA,         /* 75 */
    PIN(IOB, 4),    /* 76 */
    PIN(IOB, 7),    /* 77 */
    PIN(IOB, 6),    /* 78 */
    FOX_NA,         /* 79 */
    FOX_NA          /* 80 */
};

int update_status(pin_t fox_pins[PIN_ARRAY_SIZE])
{
    int fda, fdb, fdg;
    int mask[3];   /* IOA, IOB, IOG */
    int i, port;

    /* copy const data from the template */
    memcpy(fox_pins, fox_pins_tmpl, sizeof(pin_t)*PIN_ARRAY_SIZE);

    err_if((fda = open("/dev/gpioa", O_RDWR)) < 0);

    err_if((fdb = open("/dev/gpiob", O_RDWR)) < 0);

    err_if((fdg = open("/dev/gpiog", O_RDWR)) < 0);

    /* INPUT LINES SETUP */

    /* don't set, just read current state */
    mask[IOA] = mask[IOB] = mask[IOG] = 0;

    /* get input lines mask */
    err_if(ioctl(fda, _IO(ETRAXGPIO_IOCTYPE, IO_SETGET_INPUT), &mask[IOA]) < 0);
    err_if(ioctl(fdb, _IO(ETRAXGPIO_IOCTYPE, IO_SETGET_INPUT), &mask[IOB]) < 0);
    err_if(ioctl(fdg, _IO(ETRAXGPIO_IOCTYPE, IO_SETGET_INPUT), &mask[IOG]) < 0);

    /* set port direction into bits struct */
    for(i = 1; i <= PIN_COUNT; ++i)
    {
        port = fox_pins[i].port; /* NA, IOA, IOB or IOG */
        if(port >= 0 && (mask[port] & (1 << fox_pins[i].bit)))
            fox_pins[i].dir = DIR_INPUT;
    }

    /* get input lines 0/1 state */
    err_if(ioctl(fda, _IO(ETRAXGPIO_IOCTYPE, IO_READ_INBITS), &mask[IOA]) < 0);
    err_if(ioctl(fdb, _IO(ETRAXGPIO_IOCTYPE, IO_READ_INBITS), &mask[IOB]) < 0);
    err_if(ioctl(fdg, _IO(ETRAXGPIO_IOCTYPE, IO_READ_INBITS), &mask[IOG]) < 0);

    /* set port 3.3V or 0V state into bits struct */
    for(i = 1; i <= PIN_COUNT; ++i)
    {
        port = fox_pins[i].port; /* NA, IOA, IOB or IOG */
        if(port >= 0 && (mask[port] & (1 << fox_pins[i].bit)))
            fox_pins[i].value = 1;  /* 3.3 V */
    }

    /* OUTPUT LINES SETUP */

    /* don't set, just read current state */
    mask[IOA] = mask[IOB] = mask[IOG] = 0;

    /* get output lines mask */
    err_if(ioctl(fda,_IO(ETRAXGPIO_IOCTYPE, IO_SETGET_OUTPUT), &mask[IOA]) < 0);
    err_if(ioctl(fdb,_IO(ETRAXGPIO_IOCTYPE, IO_SETGET_OUTPUT), &mask[IOB]) < 0);
    err_if(ioctl(fdg,_IO(ETRAXGPIO_IOCTYPE, IO_SETGET_OUTPUT), &mask[IOG]) < 0);

    /* add fixed OGx lines */
    mask[IOG] |= 0x3200003e;

    /* set port direction into bits struct */
    for(i = 1; i <= PIN_COUNT; ++i)
    {
        port = fox_pins[i].port; /* NA, IOA, IOB or IOG */
        if(port >= 0 && (mask[port] & (1 << fox_pins[i].bit)))
            fox_pins[i].dir = DIR_OUTPUT;
    }

    /* get output lines 0/1 state */
    err_if(ioctl(fda, _IO(ETRAXGPIO_IOCTYPE, IO_READ_OUTBITS), &mask[IOA]) < 0);
    err_if(ioctl(fdb, _IO(ETRAXGPIO_IOCTYPE, IO_READ_OUTBITS), &mask[IOB]) < 0);
    err_if(ioctl(fdg, _IO(ETRAXGPIO_IOCTYPE, IO_READ_OUTBITS), &mask[IOG]) < 0);

    /* set port 3.3V or 0V state int bits struct */
    for(i = 1; i <= PIN_COUNT; ++i)
    {
        port = fox_pins[i].port; /* NA, IOA, IOB or IOG */
        if(port >= 0 && (mask[port] & (1 << fox_pins[i].bit)))
            fox_pins[i].value = 1;  /* 3.3 V */
        else
            fox_pins[i].value = 0;  /* 0 V */
    }


    close(fda);
    close(fdb);
    close(fdg);

    return 0;
}

int toggle_pin(int num)
{
    pin_t pin;
    int fd = -1, val = 0, mask;

    err_if(num < 1 || num > 80);

    pin = fox_pins_tmpl[num];

    /* open the I/O port */
    switch(pin.port)
    {
    case IOA:
        err_if((fd = open("/dev/gpioa", O_RDWR)) < 0);
        break;
    case IOB:
        err_if((fd = open("/dev/gpiob", O_RDWR)) < 0);
        break;
    case IOG:
        err_if((fd = open("/dev/gpiog", O_RDWR)) < 0);
        break;
    default:
        err_if("bad port");
    }

    mask = 1 << pin.bit;

    ioctl(fd, _IO(ETRAXGPIO_IOCTYPE, IO_READ_OUTBITS), &val);

    if(val & mask)
        ioctl(fd, _IO(ETRAXGPIO_IOCTYPE, IO_CLRBITS), mask);
    else
        ioctl(fd, _IO(ETRAXGPIO_IOCTYPE, IO_SETBITS), mask);

    close(fd);

    return 0;
}


